import versioneer
from setuptools import setup

setup(
    version=versioneer.get_version(),
    cmdclass=versioneer.get_cmdclass(),
    entry_points = {
        'console_scripts': ['npt=npt.cli:app'],
    }
)
